<!DOCTYPE html>
<html>
    <head>
        <title>{% block head_title %}{% endblock %}</title>
        <meta name="theme-color" content="#84b4a7">
        <style>
            body {
                background-color: #f9f9f9;
            }
            h1 {
                margin-top: 63px;
                font-family: 'Lato Black', sans-serif;
                color: #af3515;
                font-size: 30px;
            }
            p {
                font-family: 'Lato', sans-serif;
                color: #af3515;
                font-size: 15px;
            }
        </style>
        <link rel="icon" type="image/png" href="data:image/png;base64,iVBORw0KGgo=">
    </head>
    <body>
        <center>
            {% block content %}{% endblock %}
        </center>
    </body>
</html>
